"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startServer = startServer;

var _koa = _interopRequireDefault(require("koa"));

var _koaStatic = _interopRequireDefault(require("koa-static"));

var _koaEtag = _interopRequireDefault(require("koa-etag"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _path = _interopRequireDefault(require("path"));

var _historyApiFallback = require("./middleware/history-api-fallback.js");

var _babel = require("./middleware/babel.js");

var _reloadBrowser = require("./middleware/reload-browser.js");

var _compatibility = require("./middleware/compatibility.js");

var _messageChannel = require("./middleware/message-channel.js");

var _cache = require("./middleware/cache.js");

var _constants = require("./constants.js");

var _openBrowser = require("./open-browser.js");

var _createHttpServer = require("./create-http-server.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @typedef {object} EsDevServerConfig
 * @property {number} [port]
 * @property {string} [hostname]
 * @property {string} [rootDir]
 * @property {string} [appIndex]
 * @property {string[]} [moduleDirectories]
 * @property {boolean} [nodeResolve]
 * @property {boolean} [readUserBabelConfig]
 * @property {boolean} [watch]
 * @property {boolean} [openBrowser]
 * @property {string} [openPath]
 * @property {boolean} [logStartup]
 * @property {boolean} [http2]
 * @property {string} [compatibilityMode]
 * @property {string[]} [watchExcludes]
 * @property {number} [watchDebounce]
 * @property {object} [customBabelConfig]
 * @property {string[]} [extraFileExtensions]
 * @property {import('koa').Middleware[]} [customMiddlewares]
 */

/** @param {EsDevServerConfig} config */
function startServer(config) {
  const {
    port = 8080,
    hostname = '127.0.0.1',
    rootDir = process.cwd(),
    moduleDirectories = ['node_modules'],
    nodeResolve = false,
    readUserBabelConfig = false,
    customBabelConfig,
    watch = false,
    openBrowser = false,
    openPath,
    logStartup = false,
    http2 = false,
    extraFileExtensions = [],
    compatibilityMode = _constants.compatibilityModes.NONE,
    watchExcludes = ['node_modules/**'],
    watchDebounce = 1000,
    customMiddlewares
  } = config;
  let {
    appIndex
  } = config;
  let appIndexDir; // ensure appIndex is relative to rootDir

  if (appIndex) {
    if (_path.default.isAbsolute(appIndex)) {
      appIndex = `/${_path.default.relative(rootDir, appIndex)}`;
    } else if (!appIndex.startsWith('/')) {
      appIndex = `/${appIndex}`;
    }

    appIndexDir = `${appIndex.substring(0, appIndex.lastIndexOf('/'))}`;
  }

  const setupWatch = appIndex && watch;
  const setupBabel = customBabelConfig || nodeResolve || compatibilityMode !== _constants.compatibilityModes.NONE || readUserBabelConfig;
  const setupCompatibility = compatibilityMode && compatibilityMode !== _constants.compatibilityModes.NONE;
  const setupHistoryFallback = appIndex;
  const setupMessageChanel = setupWatch || setupBabel;
  const app = new _koa.default();
  /** @type {import('chokidar').FSWatcher | null} */

  const fileWatcher = setupWatch ? _chokidar.default.watch([]) : null; // add custom user's middlewares

  if (customMiddlewares && customMiddlewares.length > 0) {
    customMiddlewares.forEach(customMiddleware => {
      app.use(customMiddleware);
    });
  } // serves 304 responses if resource hasn't changed


  app.use((0, _cache.createCacheMiddleware)()); // adds etag headers for caching

  app.use((0, _koaEtag.default)()); // communicate with browser for reload or logging

  if (setupMessageChanel) {
    app.use((0, _messageChannel.createMessageChannelMiddleware)({
      rootDir,
      appIndex
    }));
  } // inject polyfills for compatibility with older browsers


  if (setupCompatibility) {
    app.use((0, _compatibility.createCompatibilityMiddleware)({
      compatibilityMode,
      appIndex,
      appIndexDir
    }));
  } // watch files and reload browser


  if (setupWatch) {
    app.use((0, _reloadBrowser.createReloadBrowserMiddleware)({
      rootDir,
      fileWatcher,
      watchExcludes,
      watchDebounce
    }));
  } // run code through babel for compatibility with older browsers


  if (setupBabel) {
    app.use((0, _babel.createBabelMiddleware)({
      rootDir,
      moduleDirectories,
      nodeResolve,
      readUserBabelConfig,
      compatibilityMode,
      extraFileExtensions,
      customBabelConfig
    }));
  } // serve index.html for non-file requests for SPA routing


  if (setupHistoryFallback) {
    app.use((0, _historyApiFallback.createHistoryAPIFallbackMiddleware)({
      appIndex,
      appIndexDir
    }));
  } // serve ststic files


  app.use((0, _koaStatic.default)(rootDir, {
    setHeaders(res) {
      res.setHeader('cache-control', 'no-cache');
    }

  })); // create http or https/http2 server

  const server = (0, _createHttpServer.createHTTPServer)(app.callback(), http2); // start the server, open the browser and log messages

  server.listen({
    port,
    host: hostname
  }, () => {
    (0, _openBrowser.openBrowserAndLogStartup)({
      appIndex,
      openBrowser,
      openPath,
      hostname,
      port,
      rootDir,
      logStartup,
      http2
    });
  }); // cleanup after quit

  server.addListener('close', () => {
    if (fileWatcher) {
      fileWatcher.close();
    }
  });
  return {
    server,
    app
  };
}